using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SnakeGame
{
    public partial class Form1 : Form
    {
        private Snake snake;
        private int thickness = 10;
        private int brushThickness = 10;
        public Panel gamePanel;
        private MyAboutBox abtbox;
        public MySettings gameSettings;

        public Form1() {
            InitializeComponent();
            gamePanel = this.pnlGame;

            gameSettings = new MySettings(
                System.IO.Path.Combine(
                    System.IO.Path.GetDirectoryName(Application.ExecutablePath),
                    "settings.xml"
                )
            );

            label7.Text = gameSettings.getSetting("HighScore") + "  (" +
                gameSettings.getSetting("HighScoreName") + ")";

            snake = new Snake(this.pnlGame.Width / 10, this.pnlGame.Height / 10, this);
            thickness = SnakeGame.Properties.Settings.Default.Thickness;
        }

        private void gameAreaPaint(object sender, PaintEventArgs e) {
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

            Brush b = null;
            if (snake.Alive) b = Brushes.Red;
            else b = Brushes.Blue;

            Point p = snake.locations[0];
            for (int i = 1; i < snake.locations.Count; i++) {
                p = snake.locations[i];
                g.FillRectangle(b, p.X * thickness, p.Y * thickness, brushThickness, brushThickness);
                //g.DrawRectangle(Pens.Black, p.X * brushThickness, p.Y * brushThickness, brushThickness, brushThickness);
            }
            p = snake.locations[0];
            g.FillRectangle(Brushes.Green, p.X * brushThickness, p.Y * brushThickness, brushThickness, brushThickness);
            g.DrawRectangle(Pens.Black, p.X * brushThickness, p.Y * brushThickness, brushThickness, brushThickness);

            g.FillEllipse(Brushes.Brown,
                snake.food.X * thickness,
                snake.food.Y * thickness,
                thickness,
                thickness);

            if (snake.specialfood.Valid) {
                if (snake.specialfood.HorizontallyOriented) {
                    g.DrawImage(snake.specialfood.Picture,
                        snake.specialfood.Location.X * thickness,
                        snake.specialfood.Location.Y * thickness,
                        20,
                        10);
                }
                else {
                    g.DrawImage(snake.specialfood.Picture,
                        snake.specialfood.Location.X * thickness,
                        snake.specialfood.Location.Y * thickness,
                        10,
                        20);
                }
            }

            
        }

        private void startToolStripMenuItem_Click(object sender, EventArgs e) {
            snake.startGame();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
            snake.running = false;
            Application.Exit();
        }


        private void KeyDownOnForm(object sender, KeyEventArgs e) {
            if (e.KeyCode == Keys.Up) {
                snake.turn(Direction.Up);
            }
            else if (e.KeyCode == Keys.Right) {
                snake.turn(Direction.Right);
            }
            else if (e.KeyCode == Keys.Left) {
                snake.turn(Direction.Left);
            }
            else if (e.KeyCode == Keys.Down) {
                snake.turn(Direction.Down);
            }

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
            //lock (snake) {
                snake.running = false;
              //  System.Threading.Monitor.Pulse(snake);
            //}
            if (snake.th!=null &&                
                (snake.th.ThreadState == System.Threading.ThreadState.Running ||
                snake.th.ThreadState == System.Threading.ThreadState.WaitSleepJoin)
                ) {
                if (!snake.th.Join(510)) {
                    snake.th.Abort();
                }
            }
        }

        public void setScoreText(string s) {
            lblScore.Text = s;
        }

        public void setSplTimeoutText(string s) {
            lblSplTimeout.Text = s;
        }

        public void setHighScoreText(string s) {
            label7.Text = s;
        }

        private void slowToolStripMenuItem_Click(object sender, EventArgs e) {
            //Uncheck everyone, then check the sender..
            slowToolStripMenuItem.CheckState = CheckState.Unchecked;
            normalToolStripMenuItem.CheckState = CheckState.Unchecked;
            fastToolStripMenuItem.CheckState = CheckState.Unchecked;
            veryFastToolStripMenuItem.CheckState = CheckState.Unchecked;

            ToolStripMenuItem t = (ToolStripMenuItem)sender;

            t.CheckState = CheckState.Checked;
            if (t == slowToolStripMenuItem) {
                snake.difficulty = 150;
                snake.scorePerBlock = 6;
                lblDifficulty.Text = "Slow";
            }
            else if (t == normalToolStripMenuItem) {
                snake.difficulty = 100;
                snake.scorePerBlock = 10;
                lblDifficulty.Text = "Normal";
            }
            else if (t == fastToolStripMenuItem) {
                snake.difficulty = 66;
                snake.scorePerBlock = 17;
                lblDifficulty.Text = "Fast";
            }
            else if (t == veryFastToolStripMenuItem) {
                lblDifficulty.Text = "Very Fast";
                snake.scorePerBlock = 25;
                snake.difficulty = 33;
            }
            
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e) {
            if (abtbox == null) {
                abtbox = new MyAboutBox();
            }
            abtbox.ShowDialog(this);
        }

        private void quitThisGameToolStripMenuItem_Click(object sender, EventArgs e) {
            snake.Alive = false;
            pnlGame.Refresh();
        }

        private void resetHighscoresToolStripMenuItem_Click(object sender, EventArgs e) {
            gameSettings.resetSettings();
            snake.highscore = 0;
            snake.highscorename = "Anonymous";
            label7.Text = gameSettings.getSetting("HighScore") + "  (" +
                gameSettings.getSetting("HighScoreName") + ")";
        }
    }

    public delegate void RefreshPanel();
    public delegate void RefreshRectPanel(Rectangle rc);
    public delegate void SetTextOnLabel(string s);

}