using System;
using System.Collections.Generic;
using System.Text;
using Point = System.Drawing.Point;
using Rectangle = System.Drawing.Rectangle;
using System.Threading;

namespace SnakeGame
{

    class Snake
    {
        #region Properties
		
        private int width;

        public int Width {
            get { return width; }
            set { width = value; }
        }

        private int height;

        public int Height {
            get { return height; }
            set { height = value; }
        }

        private bool alive = true;

        public bool Alive {
            get { return alive; }
            set { alive = value; }
        }

	    #endregion

        #region Public Variables
        public bool running = true;
        public List<Point> locations = new List<Point>();
        public Point food = new Point(-1, -1);
        public int difficulty = 100; //number of milliseconds to wait before moving one block.
        public int scorePerBlock = 10;
        public int currentscore = 0;
        public int highscore = 0;
        public string highscorename = "Anonymous";
        public Thread th;
        public SpecialFood specialfood = new SpecialFood();
        #endregion

        #region Private Variables

        private Form1 parent;
        private Random rnd = new Random();

        private int StartBlocks = 10;
        private int thickness;
        private const int waitBeforeDie = 2;
        private const int SplFoodInterval = 5;
        private const int SplFoodTimeout = 40;

        private Point oldPoint;
        private Direction direction;
        private Direction currentDirection;
        private bool NewCommandAvailable = false;
        private int cyclesWaited = 0;
        private bool firsttry = true;
        private int numFoodEaten = 0;
        #endregion
        
        #region Delegate Instantiations
        private RefreshPanel rf;
        private RefreshRectPanel rrf;
        private SetTextOnLabel stt;
        private SetTextOnLabel stthighscore;
        private SetTextOnLabel sttSplTimeout;
        private RefreshPanel updatepanel;
        #endregion


        #region Functions
        public Snake(int _width, int _height, Form1 _parent) {

            //Set initial values
            width = _width;
            height = _height;
            parent = _parent;

            currentDirection = Direction.Right;
            thickness = SnakeGame.Properties.Settings.Default.Thickness;
            highscore = SnakeGame.Properties.Settings.Default.HighScore;
            highscorename = SnakeGame.Properties.Settings.Default.HighScoreName;

            //Instantiate Delegates
            rf = new RefreshPanel(parent.gamePanel.Refresh);
            rrf = new RefreshRectPanel(parent.gamePanel.Invalidate);
            stt = new SetTextOnLabel(parent.setScoreText);
            stthighscore = new SetTextOnLabel(parent.setHighScoreText);
            updatepanel = new RefreshPanel(parent.gamePanel.Update);
            sttSplTimeout = new SetTextOnLabel(parent.setSplTimeoutText);

            //Set the initial locations of the snake
            for (int i = 0; i < StartBlocks; i++) {
                locations.Add(new Point(15 - i, 5));
            }

            checkAndUpdateHighscores();

        }

        public void startGame() {

            currentscore = 0;
            direction = currentDirection = Direction.Right;

            if (th == null) {
                running = true;
                th = new Thread(new ThreadStart(run));
                th.Start();
            }
            else {
                locations.Clear();
                for (int i = 0; i < StartBlocks; i++) {
                    locations.Add(new Point(15 - i, 5));
                }
                alive = true;
            }

            numFoodEaten = 0;
            specialfood.Valid = false;
            placeRandomFood();

            parent.Invoke(rf);
            parent.Invoke(stt, new object[] { "0" });
            parent.Invoke(sttSplTimeout, new object[] { "" });

        }

        private void placeRandomFood() {
            food = getFreePoint(false);
        }

        private Point getFreePoint(bool notOnBorder) {
            Point p = new Point();
            while (true) {
                p.X = rnd.Next(width);
                p.Y = rnd.Next(height);
                if (locations.Contains(p)) continue;
                else if (notOnBorder && ((p.X == width - 1) || (p.Y == height - 1))) continue;
                else break;
            }
            return p;
        }

        public void turn(Direction d) {
            direction = d;
            NewCommandAvailable = true;
        }

        private void run() {
            try {
                while (running) {
                    if (alive) {
                        if (NewCommandAvailable) {
                            checkAndTurn();
                            NewCommandAvailable = false;
                        }
                        if (aboutToCollide()) {
                            cyclesWaited++;
                            if (cyclesWaited == waitBeforeDie)
                                moveAhead();
                        }
                        else {
                            cyclesWaited = 0;
                            moveAhead();
                        }
                        if (checkAndEatFood()) {
                            numFoodEaten++;
                            if (numFoodEaten == SplFoodInterval) {
                                PlaceNewSpecialFood();
                                numFoodEaten = 0;
                            }
                            placeRandomFood();
                            parent.Invoke(rf);
                        }
                        Thread.Sleep(difficulty);
                    }

                    else {
                        Thread.Sleep(500);
                    }
                }
            }
            catch (ThreadAbortException) {
                //ignored
            }
        }

        private void PlaceNewSpecialFood() {
            specialfood.Location = getFreePoint(true);
            specialfood.setRandomFood();
            specialfood.TimeRemaining = SplFoodTimeout;
            specialfood.Valid = true;
        }

        private void checkAndTurn() {
            switch (currentDirection) {
                case Direction.Up:
                    if (direction != Direction.Down) currentDirection = direction;
                    break;
                case Direction.Right:
                    if (direction != Direction.Left) currentDirection = direction;
                    break;
                case Direction.Left:
                    if (direction != Direction.Right) currentDirection = direction;
                    break;
                case Direction.Down:
                    if (direction != Direction.Up) currentDirection = direction;
                    break;
            }
        }

        private void moveAhead() {
            if (specialfood.Valid) {
                specialfood.TimeRemaining--;
                parent.Invoke(sttSplTimeout, new object[] { "Time Left: "+specialfood.TimeRemaining });
                if (specialfood.TimeRemaining < 0) {
                    specialfood.Valid = false;
                    parent.Invoke(sttSplTimeout, new object[] { "" });
                    parent.Invoke(rf);
                }
            }

            Rectangle r1 = new Rectangle(
                locations[0].X*thickness-1,
                locations[0].Y*thickness-1,
                thickness+2,
                thickness+2);
            Rectangle r2 = new Rectangle(
                locations[locations.Count - 1].X * thickness-1,
                locations[locations.Count - 1].Y * thickness-1,
                thickness+2, thickness+2);
            //Compute rectangles here

            oldPoint = locations[locations.Count - 1];
            for (int i = locations.Count - 2; i >= 0 ; i--) {
                locations[i + 1] = locations[i];
            }
            locations[0] = nextPoint(locations[0], currentDirection);
            Rectangle r3 = new Rectangle(
                locations[0].X * thickness-1,
                locations[0].Y * thickness-1,
                thickness+2, thickness+2);



            //Now refresh the relevant portions of the panel. Only.
            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            //Find a method of making sure the Form is valid before making the followign calls.
            if (running) {
                if (checkCollision()) {
                    alive = false;
                    checkAndUpdateHighscores();
                    parent.Invoke(rf);
                }
                parent.Invoke(rrf, new object[] { r1 });
                parent.Invoke(rrf, new object[] { r2 });
                parent.Invoke(rrf, new object[] { r3 });
                parent.Invoke(updatepanel);
            }


        }

        private bool checkAndEatFood() {

            bool refreshrequired = false;

            if (specialfood.Valid) {
                if (specialfood.HorizontallyOriented) {
                    if (locations[0] == specialfood.Location ||
                        locations[0] == new Point(specialfood.Location.X + 1, specialfood.Location.Y)) {
                        locations.Add(oldPoint);
                        currentscore += splScore();

                        //parent.Invoke(rf);
                        refreshrequired = true;
                        parent.Invoke(sttSplTimeout, new object[] { "" });
                        parent.Invoke(stt, new object[] { currentscore.ToString() });

                        specialfood.Valid = false;
                    }
                }
                else {
                    if (locations[0] == specialfood.Location ||
                        locations[0] == new Point(specialfood.Location.X, specialfood.Location.Y + 1)) {
                        locations.Add(oldPoint);
                        currentscore += splScore();

                        //parent.Invoke(rf);
                        refreshrequired = true;
                        parent.Invoke(sttSplTimeout, new object[] { "" });
                        parent.Invoke(stt, new object[] { currentscore.ToString() });
                        
                        specialfood.Valid = false;
                    }
                }
            }

            if (locations[0] == food) {
                locations.Add(oldPoint);
                currentscore += scorePerBlock;
                parent.BeginInvoke(stt, new object[] { currentscore.ToString() });
                return true;
            }

            if (refreshrequired) {
                parent.Invoke(rf);
            }

            return false;
        }

        private int splScore() {
            return 20 + 7 * specialfood.TimeRemaining;
        }

        private bool checkCollision() {
            for (int i = 0; i < locations.Count - 1; i++) {//not really necessary - just check the head for collision!
                for (int j = i+1; j < locations.Count; j++) {
                    if (locations[i].Equals(locations[j])) return true;
                }
            }
            return false;
        }

        private bool aboutToCollide() {
            Point nextpt = nextPoint(locations[0], currentDirection);
            for (int i = 1; i < locations.Count - 1; i++) {//minus 1 since the last block will dissapear
                if (nextpt == locations[i]) return true;
            }
            return false;
        }

        private Point nextPoint(Point p, Direction d) {
            int x = p.X, y = p.Y;
            switch (d) {
                case Direction.Up:
                    y--;
                    break;
                case Direction.Right:
                    x++;
                    break;
                case Direction.Left:
                    x--;
                    break;
                case Direction.Down:
                    y++;
                    break;
                default:
                    break;
            }
            if (x >= width) x -= width;
            if (x < 0) x += width;
            if (y >= height) y -= height;
            if (y < 0) y += height;
            return new Point(x, y);
        }

        public void checkAndUpdateHighscores() {
            if (currentscore == 0) {
                //get the highscore...
                highscore = parent.gameSettings.getIntSetting("HighScore");
                highscorename = parent.gameSettings.getSetting("HighScoreName");
            }
            if (currentscore > highscore) {
                highscore = currentscore;
                highscorename = Environment.UserName;
                parent.gameSettings.setSetting("HighScore", highscore.ToString());
                parent.gameSettings.setSetting("HighScoreName", highscorename);
            }
            if(!firsttry)
                parent.Invoke(stthighscore, highscore + "  (" + highscorename + ")");
            //stthighscore(highscore + "(" + highscorename + ")");

            firsttry = false;
        }

        #endregion
    }

    public enum Direction
    {
        Up = 0,
        Right,
        Left,
        Down
    }
}
