using System;
using System.Drawing;

namespace SnakeGame
{

    /// <summary>
    /// This class encapsulates the properties of a special food
    /// </summary>
    /// <remarks>This class contains the location, time remaining, validity, type and orientation parameters for a 'special food'. A 'special food' is an object that is valid for a limited number of epochs and gives a higher bonus the faster it is gobbled up by the snake. A special food spans two snake-pixels.</remarks>
    public class SpecialFood
    {

        private const int numPictures = 4;




        private Random rnd = new Random();

        private bool valid = false;
        /// <summary>
        /// Set to true if the food is valid, that is visible and consumable.
        /// </summary>
        public bool Valid {
            get { return valid; }
            set { valid = value; }
        }


        private Point location;
        /// <summary>
        /// The location of the top, left-hand corner of the special food in snake-pixel units
        /// </summary>
        public Point Location {
            get {
                return location;
            }
            set {
                location = value;
            }
        }

        private bool horizontallyOriented;
        /// <summary>
        /// Whether the 'special food' is horizontally oriented or not
        /// </summary>
        public bool HorizontallyOriented {
            get { return horizontallyOriented; }
            set { horizontallyOriented = value; }
        }

        private int timeRemaining;
        /// <summary>
        /// Gets or sets the number of epochs remaining before this food becomes
        /// past it's expiration date :)
        /// </summary>
        public int TimeRemaining {
            get { return timeRemaining; }
            set { timeRemaining = value; }
        }


        private Bitmap picture;
        /// <summary>
        /// The picture to draw to show this special food.
        /// </summary>
        public Bitmap Picture {
            get { return picture; }
            set { picture = value; }
        }


        public void setRandomFood() {
            switch (rnd.Next(numPictures)) {
                case 0:
                    picture = SnakeGame.Properties.Resources.fish;
                    horizontallyOriented = false;
                    break;
                case 1:
                    picture = SnakeGame.Properties.Resources.worm;
                    horizontallyOriented = true;
                    break;
                case 2:
                    picture = SnakeGame.Properties.Resources.man;
                    horizontallyOriented = false;
                    break;
                case 3:
                    picture = SnakeGame.Properties.Resources.bug;
                    horizontallyOriented = true;
                    break;
                default:
                    break;
            }
        }


    }
}
